﻿using System;
using System.Collections.Generic;
using System.Linq;
using Store.BusinessLogic.Model;

namespace Store.BusinessLogic.Managers
{
    public class OrderManager : ManagerBase, IOrderManager
    {
        public OrderManager() : this(null) { }

        public OrderManager(StoreEntityModelContainer container) : base(container) { }


        #region IOrderManager Members

        public IList<Order> List(int page, int pageSize = 10)
        {
            if (page < 1)
                throw new ArgumentOutOfRangeException("page", page, "Page has to be equal to or greater than 1.");
            if (pageSize < 1)
                throw new ArgumentOutOfRangeException("pageSize", page, "Page size has to be equal to or greater than 1.");

            return DatabaseContext.Orders
                .OrderBy(o => o.Id)
                .Skip(pageSize * (page - 1)).Take(pageSize).ToList();
        }


        public IList<Order> ListUnprocessed(int page, int pageSize = 10)
        {
            if (page < 1)
                throw new ArgumentOutOfRangeException("page", page, "Page has to be equal to or greater than 1.");
            if (pageSize < 1)
                throw new ArgumentOutOfRangeException("pageSize", page, "Page size has to be equal to or greater than 1.");

            return DatabaseContext.Orders
                .Where(o => !o.IsProcessed)
                .OrderBy(o => o.Id)
                .Skip(pageSize * (page - 1))
                .Take(pageSize)
                .ToList();
        }

        public IList<Order> ListByCustomer(Customer customer, int page, int pageSize = 10)
        {
            if (customer == null)
                throw new ArgumentNullException("customer");
            if (page < 1)
                throw new ArgumentOutOfRangeException("page", page, "Page has to be equal to or greater than 1.");
            if (pageSize < 1)
                throw new ArgumentOutOfRangeException("pageSize", page, "Page size has to be equal to or greater than 1.");

            return DatabaseContext.Orders
                .Where(o => o.Customer.Id == customer.Id)
                .OrderBy(o => o.Id)
                .Skip(pageSize * (page - 1))
                .Take(pageSize)
                .ToList();
        }

        public int Count()
        {
            return DatabaseContext.Orders.Count();
        }

        public int CountUnprocessed()
        {
            return DatabaseContext.Orders.Count(o => !o.IsProcessed);
        }

        public int CountByCustomer(Customer customer)
        {
            if (customer == null)
                throw new ArgumentNullException("customer");

            return DatabaseContext.Orders.Count(o => o.Customer.Id == customer.Id);
        }


        public Order PlaceOrder(Customer customer, Product product, short quantity)
        {
            if (customer == null)
                throw new ArgumentNullException("customer");
            if (product == null)
                throw new ArgumentNullException("product");
            if (quantity < 1)
                throw new ArgumentOutOfRangeException("quantity", quantity, "Quantity has to be greater than zero.");

            Order order = new Order
                              {
                                  Customer = customer,
                                  Product = product,
                                  Quantity = quantity,
                                  DateCreated = DateTime.Now
                              };
            DatabaseContext.Orders.AddObject(order);
            return order;
        }


        public Order Get(int key)
        {
            return DatabaseContext.Orders.FirstOrDefault(o => o.Id == key);
        }


        public bool CancelOrder(Order order)
        {
            if (order == null)
                throw new ArgumentNullException("order");

            Order originalOrder = Get(order.Id);
            if (!originalOrder.IsProcessed)
            {
                DatabaseContext.Orders.DeleteObject(order);
                return true;
            }
            else
            {
                return false;
            }
        }


        public void ProcessOrders()
        {
            IQueryable<Order> ordersToProcess = DatabaseContext.Orders.Where(o => !o.IsProcessed);

            foreach (Order order in ordersToProcess)
            {
                if (order.Product.UnitsInStock >= order.Quantity)
                {
                    order.Product.UnitsInStock -= order.Quantity;
                    order.IsProcessed = true;
                }
                else
                {
                    // product je potreba oznacit jako 
                    // nedostupny a objednat u dodavatele
                }
            }
        }

        private void ValidateOrder(Order order)
        {
            if (order.Quantity < 1)
                throw new ArgumentOutOfRangeException("order.Quantity", "Order quantity must be greater than zero.");
        }

        #endregion
    }
}