﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Store.BusinessLogic.Model;
using System.Data.Objects;
using System.Linq.Expressions;

namespace Store.BusinessLogic.Managers
{
    public class ProductManager : ManagerBase, IProductManager
    {
        private readonly static Dictionary<ProductSortOrder, Func<IQueryable<Product>, IOrderedQueryable<Product>>> productSortTranslator = new Dictionary<ProductSortOrder, Func<IQueryable<Product>, IOrderedQueryable<Product>>>()
        {
            { ProductSortOrder.NameAscending, q => q.OrderBy(p => p.Name) },
            { ProductSortOrder.NameDescending, q => q.OrderBy(p => p.Name) },
            { ProductSortOrder.PriceAscending, q => q.OrderBy(p => p.Price) },
            { ProductSortOrder.PriceDescending, q => q.OrderBy(p => p.Price) }
        };

        #region Ctors

        public ProductManager() { }

        public ProductManager(StoreEntityModelContainer context) : base(context) { }

        #endregion

        #region CRUD operations

        public IEnumerable<Product> List()
        {
            return DatabaseContext.Products.OrderBy(p => p.Name).ToList();
        }


        public IEnumerable<Product> List(ProductSortOrder sortBy, int page, int pageSize = 10)
        {
            return List(null, sortBy, page, pageSize);
        }


        public IEnumerable<Product> List(string nameFilter, ProductSortOrder sortBy, int page, int pageSize = 10)
        {
            if (page < 1)
                throw new ArgumentOutOfRangeException("page", page, "Page has to be equal to or greater than 1.");
            if (pageSize < 1)
                throw new ArgumentOutOfRangeException("pageSize", page, "Page size has to be equal to or greater than 1.");

            var sortedQuery = productSortTranslator[sortBy](CreateQuery(nameFilter));
            return sortedQuery.Skip(pageSize * (page - 1)).Take(pageSize);
        }

        public int Count()
        {
            return Count(null);
        }

        public int Count(string nameFilter)
        {
            return CreateQuery(nameFilter).Count();
        }

        private ObjectQuery<Product> CreateQuery(string nameFilter)
        {
            ObjectQuery<Product> query = DatabaseContext.Products;

            if (!string.IsNullOrWhiteSpace(nameFilter))
            {
                query = query.Where("it.Name LIKE @likeParam", new ObjectParameter("likeParam", "%" + nameFilter + "%"));
            }
            return query;
        }

        public Product Get(int key)
        {
            return DatabaseContext.Products.FirstOrDefault(p => p.Id == key);
        }


        public Product Create(Product product)
        {
            ValidateProduct(product);

            DatabaseContext.Products.AddObject(product);
            return product;
        }


        public Product Create(string name, decimal price, short unitsInStock)
        {
            Product product = new Product()
            {
                Name = name,
                Price = price,
                UnitsInStock = unitsInStock
            };

            Create(product);
            return product;
        }


        public void Update(Product product)
        {
            ValidateProduct(product);

            var original = Get(product.Id);
            DatabaseContext.Products.ApplyCurrentValues(product);
        }


        public void Delete(int key)
        {
            Product productToDelete = Get(key);
            DatabaseContext.Products.DeleteObject(productToDelete);
        }


        public void Delete(Product product)
        {
            if (product == null)
                throw new ArgumentNullException("product");

            Delete(product.Id);
        }

        public bool Validate(Product product)
        {
            try
            {
                ValidateProduct(product);
                return true;
            }
            catch
            {
                return false;
            }
        }

        private void ValidateProduct(Product product)
        {
            if (product == null)
                throw new ArgumentNullException("product");
            if (product.Name == null)
                throw new ArgumentNullException("product.Name");
            if (string.IsNullOrEmpty(product.Name))
                throw new ArgumentException("Product name can't be empty.", "product.Name");
            if (product.Price < 0)
                throw new ArgumentOutOfRangeException("product.Price", product.Price, "Price must be nonnegative number.");
            if (product.UnitsInStock < 0)
                throw new ArgumentOutOfRangeException("product.UnitsInStock", product.UnitsInStock, "Units in stock must be nonnegative number.");
        }

        #endregion
    }
}
