﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Store.BusinessLogic.Managers;
using Store.BusinessLogic.Model;
using Store.Web.Models;

namespace Store.Web.Controllers
{
    [Authorize]
    public class OrderController : Controller
    {
        private const int PageSize = 4;

        private readonly IStoreUnitOfWork context = new StoreUnitOfWork();

        //
        // GET: /Order/

        public ActionResult Index(int? page)
        {
            int currentPage = page ?? 1;
            PagedListModel<Order> model = null;
            if (User.IsInRole(AccountMembershipService.CustomersRoleName))
            {
                Customer customer = context.CustomerManager.Get(User.Identity.Name);
                model = new PagedListModel<Order>
                {
                    Products = context.OrderManager.ListByCustomer(customer, currentPage),
                    Pager = new Pager(context.OrderManager.CountByCustomer(customer), currentPage, PageSize)
                };
            }
            else
            {
                model = new PagedListModel<Order>
                {
                    Products = context.OrderManager.List(currentPage),
                    Pager = new Pager(context.OrderManager.Count(), currentPage, PageSize)
                };
            }

            return View(model);
        }

        //
        // GET: /Order/Details/5

        public ActionResult Details(int id)
        {
            return View(context.OrderManager.Get(id));
        }

        //
        // GET: /Order/PlaceOrder

        public ActionResult PlaceOrder(int? productId)
        {
            var items = context.ProductManager.List() ?? new List<Product>();
            ViewBag.Products = new SelectList(items, "Id", "Name", productId);

            return View();
        }

        //
        // POST: /Order/PlaceOrder

        [HttpPost]
        public ActionResult PlaceOrder(Order order, int productId)
        {
            try
            {
                if (!ModelState.IsValid)
                {
                    return PlaceOrder(productId);
                }

                if (order.Quantity < 1)
                    ModelState.AddModelError("Quantity", "Quantity must be greater than 0.");

                Customer customer = context.CustomerManager.Get(User.Identity.Name);
                Product product = context.ProductManager.Get(productId);

                context.OrderManager.PlaceOrder(customer, product, order.Quantity);
                context.Save();

                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                return PlaceOrder(productId);
            }
        }

        //
        // GET: /Order/Cancel/5

        public ActionResult Cancel(int id)
        {
            return View(context.OrderManager.Get(id));
        }

        //
        // POST: /Order/Cancel/5

        [HttpPost]
        public ActionResult Cancel(Order order)
        {
            try
            {
                Order originalOrder = context.OrderManager.Get(order.Id);
                if (originalOrder.IsProcessed)
                {
                    ModelState.AddModelError("", "Order was already processed and can't be canceled.");
                }
                else
                {
                    context.OrderManager.CancelOrder(originalOrder);
                    context.Save();
                }
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                ModelState.AddModelError("", e);
                return View();
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                context.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}
