﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Transactions;
using System.Web;
using System.Web.Mvc;
using Store.BusinessLogic.Managers;
using Store.BusinessLogic.Model;
using Store.Web.Models;

namespace Store.Web.Controllers
{
    public class ProductController : Controller
    {
        private const int PageSize = 4;
        private readonly IStoreUnitOfWork context = new StoreUnitOfWork();

        //
        // GET: /Product/

        public ActionResult Index(string query, int? page)
        {
            int currentPage = page ?? 1;
            ViewBag.SearchQuery = query;

            var pager = new Pager(context.ProductManager.Count(query), currentPage, PageSize);
            var model = new PagedListModel<Product>
            {
                Pager = pager,
                Products = context.ProductManager.List(query, ProductSortOrder.NameAscending, pager.CurrentPage, pager.PageSize),
            };

            return View(model);
        }

        [HttpPost]
        public ActionResult Search(string query, int? page)
        {
            return RedirectToAction("Index", "Product", new { page, query });
        }

        public ActionResult Print()
        {
            return View(context.ProductManager.List());
        }

        //
        // GET: /Product/Details/5

        public ActionResult Details(int id)
        {
            return View(context.ProductManager.Get(id));
        }

        //
        // GET: /Product/Create

        [Authorize(Roles = "Administrators")]
        public ActionResult Create()
        {
            return View();
        }

        //
        // POST: /Product/Create

        [HttpPost]
        [Authorize(Roles = "Administrators")]
        public ActionResult Create(Product product)
        {
            try
            {
                if (product != null && product.UnitsInStock < 1)
                {
                    ModelState.AddModelError("UnitsInStock", "Number of units must be greater than 0.");
                    return View(product);
                }

                context.ProductManager.Create(product);
                context.Save();
                return RedirectToAction("Index");
            }
            catch
            {
                return View();
            }
        }

        //
        // GET: /Product/Edit/5

        [Authorize(Roles = "Administrators")]
        public ActionResult Edit(int id)
        {
            Product product = context.ProductManager.Get(id);
            return View(product);
        }

        //
        // POST: /Product/Edit/5

        [HttpPost]
        [Authorize(Roles = "Administrators")]
        public ActionResult Edit(Product productToEdit)
        {
            try
            {
                context.ProductManager.Update(productToEdit);
                context.Save();
                return RedirectToAction("Index");
            }
            catch
            {
                Product product = context.ProductManager.Get(productToEdit.Id);
                return View(product);
            }
        }

        //
        // GET: /Product/Delete/5

        [Authorize(Roles = "Administrators")]
        public ActionResult Delete(int id)
        {
            Product productToDelete = context.ProductManager.Get(id);
            return View(productToDelete);
        }

        //
        // POST: /Product/Delete/5

        [HttpPost]
        [Authorize(Roles = "Administrators")]
        public ActionResult Delete(Product product)
        {
            try
            {
                if (!ModelState.IsValid)
                    return View();

                context.ProductManager.Delete(product);
                context.Save();
                return RedirectToAction("Index");
            }
            catch
            {
                Product productToDelete = context.ProductManager.Get(product.Id);
                return View(productToDelete);
            }
        }

        //
        // GET: /Product/Buy/2

        [Authorize]
        public ActionResult Buy(int productId)
        {
            Product product = context.ProductManager.Get(productId);

            if (product == null)
            {
                return RedirectToAction("Index", "Product");
            }

            BuyProductModel model = new BuyProductModel() { Product = product, Quantity = 1 };
            return View(model);
        }

        [HttpPost]
        [Authorize]
        public ActionResult Buy(BuyProductModel buyModel)
        {
            try
            {
                if (!ModelState.IsValid)
                {
                    return View(buyModel);
                }

                Customer customer = context.CustomerManager.Get(User.Identity.Name);
                Product product = context.ProductManager.Get(buyModel.Product.Id);

                Order newOrder = context.OrderManager.PlaceOrder(customer, product, buyModel.Quantity);
                context.Save();

                return RedirectToAction("Details", "Order", new { id = newOrder.Id });
            }
            catch
            {
                return View(buyModel);
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                context.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}
